require_relative '../tweet'
require_relative '../io/text_reader'
require_relative 'composer'

class MarkovComposer < Composer
  def initialize(config, twitter_client, tweet_finder, emoji_finder)
    super(config, twitter_client, tweet_finder, emoji_finder)
    @chain = {}
    reader = TextReader.new(config.input_text_file)
    reader.process_input_text_file
    @beginnings = reader.beginnings
    @lines = reader.lines
    build_chain(@lines.join.split)
  end

  def compose_tweet
    text = ''
    word = @beginnings.sample
    until text.count('.') == 1 || text.length + word.length + 1 >= @max_length
      text = "#{text}#{word} "
      word = get(word)
    end
    text = clean_up(text)
    tweet = create_tweet(text)
    insert_emoji(tweet) if [true, false].sample
    tweet
  end

  def compose_image_query(query)
    "##{@chain.keys.reject { |k| @stop_words.include?(k) }.sample}"
  end

  private

  def build_chain(words)
    words.each_with_index do |word, i|
      add(word, words[i + 1]) unless i >= words.size - 2
    end
  end

  def add(word, next_word)
    @chain[word] = Hash.new(0) if @chain[word] == nil
    @chain[word][next_word] += 1
  end

  def get(word)
    return '' if @chain[word] == nil
    sum = @chain[word].reduce(0) { |sum, kv| sum += kv[1] }
    random = rand(sum) + 1
    count = 0
    @chain[word].find do |w, i|
      count += i
      count >= random
    end.first
  end

  def clean_up(text)
    text = text.strip.capitalize
    if text[text.length - 1].match('[A-Za-z]') != nil && text.length < @max_length
      text << %w(. ! ? . .).sample
    end
    text
  end

  def create_tweet(text)
    hashtags = []
    hashtags << @default_hashtag if @default_hashtag != '' && (text.length + @default_hashtag.length + 2) < @max_length
    tweet = Tweet.new(text, hashtags, @twitter_client)
    if [true, false].sample
      hashtag = @lines.sample.split.reject { |w| w.strip.nil? || /[^A-Za-z]/.match(w.strip) != nil || @stop_words.include?(w.strip) }.sample
      tweet.hashtags << hashtag if hashtag != nil && (tweet.length + hashtag.length + 2) < @max_length
    end
    tweet
  end

  def insert_emoji(tweet)
    if tweet.length < @max_length + " #{127872.chr('utf-8')}".length
      tweet.append(@emoji_finder.find)
    end
  end

  def hashtag_fits?(tweet, hashtag)
    hashtag != nil && (tweet.length + hashtag.length + 1) < @max_length
  end
end
