require_relative '../tweet'
require_relative 'composer'
require 'cgi'

class RandomComposer < Composer
  def initialize(config, twitter_client, tweet_finder, emoji_finder)
    super(config, twitter_client, tweet_finder, emoji_finder)
  end

  def compose_tweet
    tweets = @tweet_finder.find_tweets
    tweets = tweets.map { |t| normalize(t) }
    hashtag = find_random_word(tweets.sample)
    emoji = @emoji_finder.find
    text = tweets.join([', ', ' - ', '! '].sample)
    text = filter(text)
    text = trim(text, hashtag.length)
    text = replace_a_word(text.split.sample, text)
    text = replace_a_word(emoji, text)
    Tweet.new(text.capitalize, [hashtag], @twitter_client)
  end

  def compose_image_query(query)
    find_random_word(query)
  end

  private

  def find_random_word(query)
    term = extract_one_word(query)
    tweet = @tweet_finder.find_tweet(term)
    extract_one_word(tweet)
  end

  def extract_one_word(tweet)
    tweet.split.reject { |w| /[^A-Za-z0-9]/.match(w.strip) != nil || @stop_words.include?(w.strip) }.sample
  end

  def normalize(tweet)
    CGI.unescapeHTML(tweet.gsub(/(@([^\s])+|http([^\s])+|([^\s])+\/([^\s])+|)/, ''))
        .gsub(/\s+/, ' ')
        .strip
        .downcase
        .gsub(/[^[:word:]\s]/, '')
  end

  def filter(tweet)
    tweet.split.reject { |w| @blacklist.include?(w.downcase) }.join(' ')
  end

  def replace_a_word(replacement, target)
    split = target.split
    out = split.select { |w| w.length >= replacement.length }.sample
    split.map { |w| w == out ? replacement : w }.join(' ')
  end

  def trim(tweet, hashtag_length)
    if tweet.size > (@max_length - (hashtag_length > 0 ? hashtag_length + 1 : 0))
      split = tweet.split
      split.delete_at(split.length - 1)
      return trim(split.join(' '), hashtag_length)
    end
    tweet
  end
end
