module Validatable
  def valid?
    self.instance_variables.none? do |v|
      attr = instance_variable_get(v)
      attr.class.included_modules.include?(Validatable) ? !attr.valid? : attr == nil
    end
  end
end

class TwitterConfig
  include Validatable

  attr_reader :consumer_key, :consumer_secret, :access_token, :access_token_secret,
              :username, :tweet_max_length

  def initialize(consumer_key, consumer_secret, access_token, access_token_secret, username, tweet_max_length)
    @consumer_key = consumer_key
    @consumer_secret = consumer_secret
    @access_token = access_token
    @access_token_secret = access_token_secret
    @username = username
    @tweet_max_length = tweet_max_length
  end
end

class FlickrConfig
  include Validatable

  attr_reader :api_key, :shared_secret

  def initialize(api_key, shared_secret)
    @api_key = api_key
    @shared_secret = shared_secret
  end
end

class TweetomatorConfig
  include Validatable

  attr_reader :twitter_config, :flickr_config, :topics, :blacklist, :stop_words, :delay_in_seconds, :composer_type, :input_text_file, :default_hashtag

  def initialize(twitter_config, flickr_config, topics, blacklist, stop_words, delay_in_seconds, composer_type, input_text_file, default_hashtag='')
    @twitter_config = twitter_config
    @flickr_config = flickr_config
    @topics = topics
    @blacklist = blacklist
    @stop_words = stop_words
    @delay_in_seconds = delay_in_seconds
    @composer_type = composer_type
    @input_text_file = input_text_file
    @default_hashtag = default_hashtag
  end
end
