require 'tweetomator/version'
require 'twitter'
require 'rufus-scheduler'
require_relative 'tweet'
require_relative 'composers/random_composer'
require_relative 'composers/markov_composer'
require_relative 'config/config'
require_relative 'finders/tweet_finder'
require_relative 'finders/flickr_findr'
require_relative 'finders/emoji_finder'
require_relative 'clients/flickr_client'
require_relative 'clients/twitter_client'
require_relative 'io/downloader'

class Tweetomator
  def initialize(tweetomator_config)
    if tweetomator_config.valid?
      somebody_set_us_up_the_bot(tweetomator_config)
    else
      raise 'Invalid TweetomatorConfig'
    end
  end

  def run
    begin
      @scheduler.interval("#{@delay}s") { do_stuff }
      @scheduler.join
    rescue
      retry
    end
  end

  def run_once
    do_stuff
  end

  private

  def somebody_set_us_up_the_bot(config)
    @delay = config.delay_in_seconds
    @twitter_client = TwitterClient.new(config.twitter_config)
    tweet_finder = TweetFinder.new(@twitter_client, config.topics)
    flickr_client = FlickrClient.new(config.flickr_config)
    @flickr_finder = FlickrFindr.new(flickr_client)
    emoji_finder = EmojiFinder.new
    composer_class = Object.const_get(config.composer_type)
    @composer = composer_class.new(config, @twitter_client, tweet_finder, emoji_finder)
    @downloader = Downloader.new
    @scheduler = Rufus::Scheduler.new
  end

  def do_stuff
    r = (1..9).to_a.sample
    if r < 4
      tweet_with_image!
    elsif r < 7
      tweet!
    else
      follow!
    end
  end

  def tweet!
    tweet = @composer.compose_tweet
    tweet.post!
  end

  def tweet_with_image!
    tweet = @composer.compose_tweet
    flickr_query = @composer.compose_image_query(tweet.hashtags.sample)
    image_uri = @flickr_finder.find(flickr_query)
    image_local = @downloader.download(image_uri, 'image')
    tweet.post!(image_local)
  end

  def follow!
    user = @twitter_client.random_follower_of_random_follower
    @twitter_client.follow(user) if user
  end
end
